############################################
#
# Split layers by field.
# Developed by James Stott @ Nicholas Pearson Associates
# www.npaconsult.co.uk
# 21/08/2012
# Based on the code of the Split Shapefile plugin.
#
###############################################

# -*- coding: utf-8 -*-

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.core import *
from qgis.gui import *

from layersbyfielddialogbase import Ui_LayersByFieldDialog

class layers_by_field_dialog( QDialog, Ui_LayersByFieldDialog ):
  def __init__( self, iface ):
    QDialog.__init__( self )
    self.setupUi( self )
    self.iface = iface
    self.btnOk = self.buttonBox.button( QDialogButtonBox.Ok )

    QObject.connect( self.inputLayerCombo, SIGNAL( "currentIndexChanged( QString )" ), self.updateFieldList )

    self.manageGui()

  def manageGui( self ):
    self.inputLayerCombo.clear()
    self.splitFieldCombo.clear()    
    layers = self.getVectorLayers()
    self.inputLayerCombo.addItems( layers )

  def updateFieldList( self, inputLayer ):
    if self.inputLayerCombo.currentIndex() >= 1:
        self.splitFieldCombo.clear()
        layer = self.getLayerByName( inputLayer )
        fields = self.getFields(layer)
        for i in fields:
          self.splitFieldCombo.addItem( unicode( fields[i].name() ) )

  def reject( self ):
    QDialog.reject( self )

  def accept( self ):
    if self.inputLayerCombo.currentText() == "":
      QMessageBox.information( self, self.tr( "Layers From Field" ), self.tr( "No input layer specified" ) )
    else:
      inField = self.splitFieldCombo.currentText()
      inLayer = self.getLayerByName( unicode( self.inputLayerCombo.currentText() ) )

      self.btnOk.setEnabled( True )

      self.split( inLayer, inField )

      self.reject()
      
  def getVectorLayers( self ):
    layerMap = QgsMapLayerRegistry.instance().mapLayers()
    layerList = []
    layerList.append("")
    for name, layer in layerMap.iteritems():
      if layer.type() == QgsMapLayer.VectorLayer:
        layerList.append( unicode( layer.name() ) )
    return layerList

  def getLayerByName( self, layerName ):
    layerMap = QgsMapLayerRegistry.instance().mapLayers()
    for name, layer in layerMap.iteritems():
      if layer.type() == QgsMapLayer.VectorLayer and layer.name() == layerName:
        if layer.isValid():
          return layer
        else:
          return None

  def getFields(self,vLayer):
    index = 0
    vProvider = vLayer.dataProvider()
        
    layer_fields = {} 
    idx = 0
    field_idx = 0
    for k,v in vProvider.fields().iteritems():
                
        try:
            v = vProvider.fields()[k]
               
        except:
            v = None
    
        layer_fields[idx] = v
        idx = idx + 1   
        
    return layer_fields   

  def split( self, layer, field ):
    
    index = 0
    vProvider = layer.dataProvider()
    
    layer_fields = {} 
    idx = 0
    field_idx = 0
    for k,v in vProvider.fields().iteritems():
        
        if str(v.name()) == field:
            field_idx = idx
            
        try:
            v = vProvider.fields()[k]
            
        except:
            v = None

        layer_fields[idx] = v
        idx = idx + 1      

    uniques = vProvider.uniqueValues(field_idx)
    allAttrs = vProvider.attributeIndexes()
    vProvider.select( allAttrs )

    # get uniquie values in field
    uValues = []
    ft = QgsFeature()
    while vProvider.nextFeature( ft ):
      atMap = ft.attributeMap()
      if atMap.values()[ field_idx ].toString() not in uValues:
        uValues.append( atMap.values()[ field_idx ].toString() )

    self.progressBar.setRange( 0, len( uValues ) )
    
    j = 0
    
    for i in uValues:
        
        if str(vProvider.name()) == "postgres":
        
            self.vlayer = QgsVectorLayer(vProvider.dataSourceUri(), str(layer.name()) + "_" + str(uValues[j]), "postgres")
        
        elif str(vProvider.name()) == "ogr":
        
            self.vlayer = QgsVectorLayer(vProvider.dataSourceUri(), str(layer.name()) + "_" + str(uValues[j]), "ogr")

        sqlstring = field + " = '" + str(uValues[j]) + "'"
        j = j+1
        self.vlayer.setSubsetString( sqlstring )
        QgsMapLayerRegistry.instance().addMapLayer(self.vlayer)
        self.progressBar.setValue( self.progressBar.value() + 1 )        

